import { defineComponent, computed, createElementBlock, openBlock, createElementVNode, normalizeClass, unref, createCommentVNode, withModifiers, renderSlot, toDisplayString, Fragment, renderList, createTextVNode } from "vue";
import useClasses from "../../composables/useClasses/index.js";
const _hoisted_1 = ["aria-label", "aria-controls"], _hoisted_2 = ["aria-label"], _hoisted_3 = {
  key: 0,
  "aria-hidden": "true"
}, _hoisted_4 = { key: 1 }, _hoisted_5 = ["aria-label"], _hoisted_6 = { "aria-hidden": "true" }, _hoisted_7 = ["onClick", "aria-label"], _hoisted_8 = { class: "page-link" }, _hoisted_9 = ["aria-label"], _hoisted_10 = { "aria-hidden": "true" }, _hoisted_11 = ["aria-label"], _hoisted_12 = {
  key: 0,
  "aria-hidden": "true"
}, _hoisted_13 = { key: 1 }, _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "Pagination",
  props: {
    /**
     * Current page number
     */
    modelValue: {
      type: [Number, String, Boolean],
      default: null
    },
    /**
     * Total number of rows in the dataset
     */
    totalRows: {
      type: [Number, String],
      default: 0
    },
    /**
     * Number of rows per page
     */
    perPage: {
      type: [Number, String],
      default: 20
    },
    /**
     * Maximum number of buttons to show
     */
    limit: {
      type: [Number, String],
      default: 5
    },
    /**
     * Alignment of the page buttons
     */
    align: {
      type: String,
      default: "left",
      validator: (value) => ["left", "right", "center", "fill", "start", "end"].includes(value)
    },
    /**
     * Size of the rendered buttons
     */
    size: {
      type: String,
      default: "",
      validator: (value) => ["", "sm", "md", "lg"].includes(value)
    },
    /**
     * Applies pill styling to the pagination buttons
     */
    pills: {
      type: Boolean,
      default: !1
    },
    /**
     * When set to true, disables the component's functionality
     */
    disabled: {
      type: Boolean,
      default: !1
    },
    /**
     * Hides the goto first and goto last page buttons
     */
    hideGotoEndButtons: {
      type: Boolean,
      default: !1
    },
    /**
     * Do not show ellipsis buttons
     */
    hideEllipsis: {
      type: Boolean,
      default: !1
    },
    /**
     * Display first page number instead of Goto First button
     */
    firstNumber: {
      type: Boolean,
      default: !1
    },
    /**
     * Display last page number instead of Goto Last button
     */
    lastNumber: {
      type: Boolean,
      default: !1
    },
    /**
     * Content to place in the goto first page button
     */
    firstText: {
      type: String,
      default: "«"
    },
    /**
     * Content to place in the goto previous page button
     */
    prevText: {
      type: String,
      default: "‹"
    },
    /**
     * Content to place in the goto next page button
     */
    nextText: {
      type: String,
      default: "›"
    },
    /**
     * Content to place in the goto last page button
     */
    lastText: {
      type: String,
      default: "»"
    },
    /**
     * Content to place in the ellipsis placeholder
     */
    ellipsisText: {
      type: String,
      default: "…"
    },
    /**
     * Value to place in the 'aria-label' attribute of the pagination control
     */
    ariaLabel: {
      type: String,
      default: "Pagination"
    },
    /**
     * If this component controls another component or element, set this to the ID
     */
    ariaControls: {
      type: String,
      default: ""
    },
    /**
     * Value to place in the 'aria-label' attribute of the goto first page button
     */
    labelFirstPage: {
      type: String,
      default: "Go to first page"
    },
    /**
     * Value to place in the 'aria-label' attribute of the goto previous page button
     */
    labelPrevPage: {
      type: String,
      default: "Go to previous page"
    },
    /**
     * Value to place in the 'aria-label' attribute of the goto page button
     */
    labelPage: {
      type: [Function, String],
      default: "Go to page"
    },
    /**
     * Value to place in the 'aria-label' attribute of the goto next page button
     */
    labelNextPage: {
      type: String,
      default: "Go to next page"
    },
    /**
     * Value to place in the 'aria-label' attribute of the goto last page button
     */
    labelLastPage: {
      type: String,
      default: "Go to last page"
    },
    /**
     * Class(es) to apply to the 'Go to first page' button
     */
    firstClass: {
      type: [Array, Object, String],
      default: ""
    },
    /**
     * Class(es) to apply to the 'Go to previous page' button
     */
    prevClass: {
      type: [Array, Object, String],
      default: ""
    },
    /**
     * Class(es) to apply to the 'Go to page #' buttons
     */
    pageClass: {
      type: [Array, Object, String],
      default: ""
    },
    /**
     * Class(es) to apply to the 'Go to next page' button
     */
    nextClass: {
      type: [Array, Object, String],
      default: ""
    },
    /**
     * Class(es) to apply to the 'Go to last page' button
     */
    lastClass: {
      type: [Array, Object, String],
      default: ""
    },
    /**
     * Class(es) to apply to the 'ellipsis' placeholders
     */
    ellipsisClass: {
      type: [Array, Object, String],
      default: ""
    }
  },
  emits: ["update:modelValue", "change", "input", "page-click"],
  setup(__props, { emit: __emit }) {
    const props = __props, emit = __emit, currentPage = computed(() => {
      const value = props.modelValue;
      return typeof value == "boolean" ? 1 : Number(value) || 1;
    }), totalPages = computed(() => {
      const total = Number(props.totalRows), perPage = Number(props.perPage);
      return Math.ceil(total / perPage) || 1;
    }), displayedPages = computed(() => {
      const pages = [], limit = Number(props.limit), halfLimit = Math.floor(limit / 2);
      let start = Math.max(1, currentPage.value - halfLimit), end = Math.min(totalPages.value, start + limit - 1);
      end === totalPages.value && (start = Math.max(1, end - limit + 1)), start > 1 && (pages.push(1), start > 2 && !props.hideEllipsis && pages.push("..."));
      for (let i = start; i <= end; i++)
        pages.push(i);
      return end < totalPages.value && (end < totalPages.value - 1 && !props.hideEllipsis && pages.push("..."), pages.push(totalPages.value)), pages;
    }), onPageClick = (page) => {
      if (!props.disabled && page >= 1 && page <= totalPages.value && page !== currentPage.value) {
        const bvEvent = {
          preventDefault: () => {
          }
        };
        emit("page-click", bvEvent, page), bvEvent.preventDefault || (emit("update:modelValue", page), emit("change", page), emit("input", page));
      }
    }, { classes } = useClasses(computed(() => [
      props.size ? `pagination-${props.size}` : void 0,
      props.pills ? "pagination-pills" : void 0,
      props.align === "fill" ? "pagination-fill" : void 0,
      ["start", "left"].includes(props.align) ? "justify-content-start" : void 0,
      props.align === "center" ? "justify-content-center" : void 0,
      ["end", "right"].includes(props.align) ? "justify-content-end" : void 0
    ]));
    return (_ctx, _cache) => (openBlock(), createElementBlock("nav", {
      "aria-label": __props.ariaLabel,
      "aria-controls": __props.ariaControls
    }, [
      createElementVNode("ul", {
        class: normalizeClass(["pagination", unref(classes)])
      }, [
        __props.hideGotoEndButtons ? createCommentVNode("", !0) : (openBlock(), createElementBlock("li", {
          key: 0,
          class: normalizeClass(["page-item", [__props.firstClass, { disabled: __props.disabled || currentPage.value === 1 }]])
        }, [
          createElementVNode("a", {
            class: "page-link",
            href: "#",
            onClick: _cache[0] || (_cache[0] = withModifiers(($event) => onPageClick(1), ["prevent"])),
            "aria-label": __props.labelFirstPage
          }, [
            renderSlot(_ctx.$slots, "first-text", {}, () => [
              __props.firstNumber ? (openBlock(), createElementBlock("span", _hoisted_4, "1")) : (openBlock(), createElementBlock("span", _hoisted_3, toDisplayString(__props.firstText), 1))
            ])
          ], 8, _hoisted_2)
        ], 2)),
        createElementVNode("li", {
          class: normalizeClass(["page-item", [__props.prevClass, { disabled: __props.disabled || currentPage.value === 1 }]])
        }, [
          createElementVNode("a", {
            class: "page-link",
            href: "#",
            onClick: _cache[1] || (_cache[1] = withModifiers(($event) => onPageClick(currentPage.value - 1), ["prevent"])),
            "aria-label": __props.labelPrevPage
          }, [
            renderSlot(_ctx.$slots, "prev-text", {}, () => [
              createElementVNode("span", _hoisted_6, toDisplayString(__props.prevText), 1)
            ])
          ], 8, _hoisted_5)
        ], 2),
        (openBlock(!0), createElementBlock(Fragment, null, renderList(displayedPages.value, (page, index) => (openBlock(), createElementBlock(Fragment, { key: page }, [
          page !== "..." ? (openBlock(), createElementBlock("li", {
            key: 0,
            class: normalizeClass(["page-item", [__props.pageClass, { active: page === currentPage.value }]])
          }, [
            createElementVNode("a", {
              class: "page-link",
              href: "#",
              onClick: withModifiers(($event) => onPageClick(page), ["prevent"]),
              "aria-label": typeof __props.labelPage == "function" ? __props.labelPage(page) : `${__props.labelPage} ${page}`
            }, [
              renderSlot(_ctx.$slots, "page", { page }, () => [
                createTextVNode(toDisplayString(page), 1)
              ])
            ], 8, _hoisted_7)
          ], 2)) : (openBlock(), createElementBlock("li", {
            key: 1,
            class: normalizeClass(["page-item disabled", __props.ellipsisClass])
          }, [
            createElementVNode("span", _hoisted_8, [
              renderSlot(_ctx.$slots, "ellipsis-text", {}, () => [
                createTextVNode(toDisplayString(__props.ellipsisText), 1)
              ])
            ])
          ], 2))
        ], 64))), 128)),
        createElementVNode("li", {
          class: normalizeClass(["page-item", [__props.nextClass, { disabled: __props.disabled || currentPage.value === totalPages.value }]])
        }, [
          createElementVNode("a", {
            class: "page-link",
            href: "#",
            onClick: _cache[2] || (_cache[2] = withModifiers(($event) => onPageClick(currentPage.value + 1), ["prevent"])),
            "aria-label": __props.labelNextPage
          }, [
            renderSlot(_ctx.$slots, "next-text", {}, () => [
              createElementVNode("span", _hoisted_10, toDisplayString(__props.nextText), 1)
            ])
          ], 8, _hoisted_9)
        ], 2),
        __props.hideGotoEndButtons ? createCommentVNode("", !0) : (openBlock(), createElementBlock("li", {
          key: 1,
          class: normalizeClass(["page-item", [__props.lastClass, { disabled: __props.disabled || currentPage.value === totalPages.value }]])
        }, [
          createElementVNode("a", {
            class: "page-link",
            href: "#",
            onClick: _cache[3] || (_cache[3] = withModifiers(($event) => onPageClick(totalPages.value), ["prevent"])),
            "aria-label": __props.labelLastPage
          }, [
            renderSlot(_ctx.$slots, "last-text", {}, () => [
              __props.lastNumber ? (openBlock(), createElementBlock("span", _hoisted_13, toDisplayString(totalPages.value), 1)) : (openBlock(), createElementBlock("span", _hoisted_12, toDisplayString(__props.lastText), 1))
            ])
          ], 8, _hoisted_11)
        ], 2))
      ], 2)
    ], 8, _hoisted_1));
  }
});
export {
  _sfc_main as default
};
//# sourceMappingURL=Pagination.vue.js.map
