import { defineComponent, ref, computed, createElementBlock, openBlock, createCommentVNode, createElementVNode, withDirectives, unref, vModelText, normalizeStyle, normalizeClass, renderSlot, toDisplayString, Fragment, renderList, createTextVNode, mergeProps, toHandlers, createBlock, Transition, withCtx } from "vue";
import useClasses from "../../composables/useClasses/index.js";
import "../../composables/useFormControl/index.js";
import "uuid";
import debounce from "../../../node_modules/lodash-es/debounce.js";
const _hoisted_1 = {
  key: 0,
  class: "mb-3"
}, _hoisted_2 = ["placeholder"], _hoisted_3 = {
  key: 0,
  class: "table-busy-overlay"
}, _hoisted_4 = ["id"], _hoisted_5 = ["innerHTML"], _hoisted_6 = { key: 1 }, _hoisted_7 = { key: 1 }, _hoisted_8 = { key: 2 }, _hoisted_9 = ["onClick"], _hoisted_10 = ["colspan"], _hoisted_11 = ["onClick", "onDblclick", "onContextmenu", "onMouseenter", "onMouseleave", "onMousedown"], _hoisted_12 = ["colspan"], _hoisted_13 = { key: 3 }, _hoisted_14 = ["onClick"], _hoisted_15 = {
  key: 1,
  class: "d-flex justify-content-between align-items-center mt-3"
}, _hoisted_16 = { class: "d-flex align-items-center" }, _hoisted_17 = ["value"], _hoisted_18 = ["value"], _hoisted_19 = { class: "d-flex align-items-center" }, _hoisted_20 = ["value"], _hoisted_21 = ["value"], _hoisted_22 = { class: "ms-2" }, _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "Table",
  props: {
    /**
     * The fields of the table
     */
    fields: {
      type: Array,
      required: !1,
      default: void 0
    },
    /**
     * The items that are shown in the table
     */
    items: {
      type: Array,
      default: () => []
    },
    /**
     * Defines if multiple items can be sorted at once
     */
    multiSort: {
      type: Boolean,
      default: !1
    },
    /**
     * Defines if the `table-hover` class should be used
     */
    hover: {
      type: Boolean,
      default: !1
    },
    /**
     * Defines if the `table-click` class should be used
     */
    // $attrs doesn't return emitted events, so
    // we have to catch it as a prop
    // eslint-disable-next-line vue/prop-name-casing
    "onClick:row": {
      type: Function,
      default: void 0
    },
    /**
     * The classes that will be forwarded to the tbody tr
     */
    tbodyTrClass: {
      type: [String, Array, Object, Function],
      default: void 0
    },
    /**
     * The classes that will be forwarded to the tbody tr
     */
    theadTrClass: {
      type: [String, Array, Object],
      default: void 0
    },
    /**
     * The ID of the table
     */
    id: {
      type: String,
      default: void 0
    },
    /**
     * The caption of the table
     */
    caption: {
      type: String,
      default: void 0
    },
    /**
     * The HTML caption of the table
     */
    captionHtml: {
      type: String,
      default: void 0
    },
    /**
     * Whether the caption should be on top
     */
    captionTop: {
      type: Boolean,
      default: !1
    },
    /**
     * The text to show when there are no items
     */
    emptyText: {
      type: String,
      default: "There are no records to show"
    },
    /**
     * The text to show when there are no filtered items
     */
    emptyFilteredText: {
      type: String,
      default: "There are no records matching your request"
    },
    /**
     * Whether to show the footer
     */
    footClone: {
      type: Boolean,
      default: !1
    },
    /**
     * Whether to disable footer sorting
     */
    noFooterSorting: {
      type: Boolean,
      default: !1
    },
    /**
     * The variant for the header row
     */
    headRowVariant: {
      type: String,
      default: void 0
    },
    /**
     * The variant for the footer row
     */
    footRowVariant: {
      type: String,
      default: void 0
    },
    /**
     * The class for the tbody
     */
    tbodyClass: {
      type: [String, Array, Object],
      default: void 0
    },
    /**
     * The class for the tfoot tr
     */
    tfootTrClass: {
      type: [String, Array, Object],
      default: void 0
    },
    /**
     * Whether the table is selectable
     */
    selectable: {
      type: Boolean,
      default: !1
    },
    /**
     * The select mode
     */
    selectMode: {
      type: String,
      default: "multi",
      validator: (value) => ["single", "multi", "range"].includes(value)
    },
    /**
     * The variant for selected rows
     */
    selectedVariant: {
      type: String,
      default: "active"
    },
    /**
     * Whether to show row details
     */
    showDetails: {
      type: Boolean,
      default: !1
    },
    /**
     * The class for the details td
     */
    detailsTdClass: {
      type: [String, Array, Object],
      default: void 0
    },
    /**
     * Number of rows to show per page
     */
    perPage: {
      type: [Number, String],
      default: 0
    },
    /**
     * Available options for per page
     */
    perPageOptions: {
      type: Array,
      default: () => [10, 25, 50, 100]
    },
    /**
     * Current page number
     */
    currentPage: {
      type: [Number, String],
      default: 1
    },
    /**
     * Total number of rows
     */
    totalRows: {
      type: [Number, String],
      default: 0
    },
    /**
     * Whether to use internal pagination
     */
    noProviderPaging: {
      type: Boolean,
      default: !1
    },
    /**
     * Whether the table is filterable
     */
    filterable: {
      type: Boolean,
      default: !1
    },
    /**
     * Placeholder text for the filter input
     */
    filterPlaceholder: {
      type: String,
      default: "Filter..."
    },
    /**
     * Debounce time for filter input
     */
    filterDebounce: {
      type: [Number, String],
      default: 0
    },
    /**
     * Custom filter function
     */
    filterFunction: {
      type: Function,
      default: void 0
    },
    /**
     * Fields to ignore when filtering
     */
    filterIgnoredFields: {
      type: Array,
      default: () => []
    },
    /**
     * Fields to include when filtering
     */
    filterIncludedFields: {
      type: Array,
      default: () => []
    },
    /**
     * Whether to use internal filtering
     */
    noProviderFiltering: {
      type: Boolean,
      default: !1
    },
    /**
     * Makes the table responsive
     */
    responsive: {
      type: [Boolean, String],
      default: !1,
      validator: (value) => typeof value == "boolean" ? !0 : ["sm", "md", "lg", "xl"].includes(value)
    },
    /**
     * Makes the table header sticky
     */
    stickyHeader: {
      type: [Boolean, String],
      default: !1
    },
    /**
     * Whether the table is in busy state
     */
    busy: {
      type: Boolean,
      default: !1
    },
    /**
     * Transition props for tbody
     */
    tbodyTransitionProps: {
      type: Object,
      default: () => ({})
    },
    /**
     * Transition handlers for tbody
     */
    tbodyTransitionHandlers: {
      type: Object,
      default: () => ({})
    },
    /**
     * Transition props for row details
     */
    detailsTransitionProps: {
      type: Object,
      default: () => ({
        name: "fade",
        mode: "out-in"
      })
    },
    /**
     * Transition handlers for row details
     */
    detailsTransitionHandlers: {
      type: Object,
      default: () => ({})
    }
  },
  emits: ["click:row", "hover:row", "sort", "row-dblclicked", "row-contextmenu", "row-hovered", "row-unhovered", "row-middle-clicked", "row-selected", "update:currentPage", "update:perPage", "pagination", "filtered"],
  setup(__props, { emit: __emit }) {
    const props = __props, emit = __emit, sorted = ref(
      (props.fields || []).filter((f) => f.sortable && f.sort).reduce((obj, item) => Object.assign(obj, { [item.key]: item.sort }), {})
    ), selectedRows = ref([]), headers = computed(() => {
      if (props.fields)
        return props.fields;
      const first = props.items.find((i) => i !== void 0);
      return first ? Object.keys(first).map((f) => ({ key: f })) : [];
    }), sort = (field) => {
      if (!field.sortable)
        return;
      const key = field.key, sort2 = sorted.value[key];
      props.multiSort ? sort2 === "asc" ? sorted.value[key] = "desc" : sort2 === "desc" ? delete sorted.value[key] : sorted.value[key] = "asc" : sorted.value = {
        [key]: sort2 === "asc" ? "desc" : "asc"
      }, emit("sort", sorted.value);
    }, onRowClick = (item, index, event) => {
      if (props.selectable) {
        if (props.selectMode === "single")
          selectedRows.value = [item];
        else if (props.selectMode === "multi") {
          const idx = selectedRows.value.indexOf(item);
          idx === -1 ? selectedRows.value.push(item) : selectedRows.value.splice(idx, 1);
        } else props.selectMode;
        emit("row-selected", selectedRows.value);
      }
      emit("click:row", item);
    }, { classes } = useClasses(computed(() => [
      props.hover && "table-hover",
      props["onClick:row"] && "table-click",
      props.stickyHeader && "table-sticky-header",
      props.busy && "table-busy"
    ])), totalPages = computed(() => {
      const total = Number(props.totalRows), perPage = Number(props.perPage);
      return Math.ceil(total / perPage) || 1;
    }), paginatedItems = computed(() => {
      if (!props.noProviderPaging || props.perPage <= 0)
        return props.items;
      const start = (Number(props.currentPage) - 1) * Number(props.perPage), end = start + Number(props.perPage);
      return props.items.slice(start, end);
    }), filterValue = ref(""), filterItems = (items) => {
      if (!props.noProviderFiltering || !filterValue.value)
        return items;
      const searchValue = filterValue.value.toLowerCase(), fields = props.filterIncludedFields.length > 0 ? props.filterIncludedFields : Object.keys(items[0] || {}).filter((field) => !props.filterIgnoredFields.includes(field));
      return props.filterFunction ? props.filterFunction(items, searchValue) : items.filter((item) => fields.some((field) => {
        const value = item[field];
        return value == null ? !1 : String(value).toLowerCase().includes(searchValue);
      }));
    }, onFilterInput = debounce(() => {
      const filtered = filterItems(props.items);
      emit("filtered", filtered);
    }, Number(props.filterDebounce)), filteredItems = computed(() => filterItems(props.items)), displayItems = computed(() => {
      const items = props.noProviderFiltering ? filteredItems.value : props.items;
      return props.noProviderPaging ? paginatedItems.value : items;
    }), stickyHeaderStyle = computed(() => props.stickyHeader ? {
      maxHeight: typeof props.stickyHeader == "string" ? props.stickyHeader : "300px",
      overflowY: "auto"
    } : {}), onPerPageChange = (event) => {
      const value = Number(event.target.value);
      emit("update:perPage", value), emit("pagination", {
        currentPage: Number(props.currentPage),
        perPage: value
      });
    }, onPageChange = (event) => {
      const value = Number(event.target.value);
      emit("update:currentPage", value), emit("pagination", {
        currentPage: value,
        perPage: Number(props.perPage)
      });
    };
    return (_ctx, _cache) => (openBlock(), createElementBlock("div", null, [
      __props.filterable ? (openBlock(), createElementBlock("div", _hoisted_1, [
        withDirectives(createElementVNode("input", {
          "onUpdate:modelValue": _cache[0] || (_cache[0] = ($event) => filterValue.value = $event),
          type: "text",
          class: "form-control",
          placeholder: __props.filterPlaceholder,
          onInput: _cache[1] || (_cache[1] = //@ts-ignore
          (...args) => unref(onFilterInput) && unref(onFilterInput)(...args))
        }, null, 40, _hoisted_2), [
          [vModelText, filterValue.value]
        ])
      ])) : createCommentVNode("", !0),
      createElementVNode("div", {
        class: normalizeClass([
          "table-responsive",
          {
            "table-responsive-sm": __props.responsive === "sm",
            "table-responsive-md": __props.responsive === "md",
            "table-responsive-lg": __props.responsive === "lg",
            "table-responsive-xl": __props.responsive === "xl"
          }
        ]),
        style: normalizeStyle(stickyHeaderStyle.value)
      }, [
        __props.busy ? (openBlock(), createElementBlock("div", _hoisted_3, [
          renderSlot(_ctx.$slots, "table-busy", {}, () => [
            _cache[2] || (_cache[2] = createElementVNode("div", { class: "table-busy-spinner" }, [
              createElementVNode("div", {
                class: "spinner-border",
                role: "status"
              }, [
                createElementVNode("span", { class: "visually-hidden" }, "Loading...")
              ])
            ], -1))
          ])
        ])) : createCommentVNode("", !0),
        createElementVNode("table", {
          class: normalizeClass(["table", unref(classes)]),
          id: __props.id
        }, [
          __props.caption || __props.captionHtml ? (openBlock(), createElementBlock("caption", {
            key: 0,
            class: normalizeClass({ "caption-top": __props.captionTop })
          }, [
            __props.captionHtml ? (openBlock(), createElementBlock("span", {
              key: 0,
              innerHTML: __props.captionHtml
            }, null, 8, _hoisted_5)) : (openBlock(), createElementBlock("span", _hoisted_6, toDisplayString(__props.caption), 1))
          ], 2)) : createCommentVNode("", !0),
          _ctx.$slots["table-colgroup"] ? (openBlock(), createElementBlock("colgroup", _hoisted_7, [
            renderSlot(_ctx.$slots, "table-colgroup")
          ])) : createCommentVNode("", !0),
          headers.value.length ? (openBlock(), createElementBlock("thead", _hoisted_8, [
            createElementVNode("tr", {
              class: normalizeClass([__props.theadTrClass, __props.headRowVariant ? `table-${__props.headRowVariant}` : ""])
            }, [
              renderSlot(_ctx.$slots, "thead-top"),
              (openBlock(!0), createElementBlock(Fragment, null, renderList(headers.value, (header) => renderSlot(_ctx.$slots, `header-${header.key}`, {
                key: `header-${header.key}`,
                header
              }, () => [
                (openBlock(), createElementBlock("th", {
                  key: header.key,
                  class: normalizeClass({
                    "table-heading-sortable": header.sortable,
                    "table-heading-sortable-asc": sorted.value[header.key] === "asc",
                    "table-heading-sortable-desc": sorted.value[header.key] === "desc"
                  }),
                  onClick: ($event) => sort(header)
                }, [
                  renderSlot(_ctx.$slots, `header-${header.key}-content`, { header }, () => [
                    createTextVNode(toDisplayString(header.label || header.key), 1)
                  ])
                ], 10, _hoisted_9))
              ])), 128))
            ], 2)
          ])) : createCommentVNode("", !0),
          createElementVNode("tbody", mergeProps({ class: __props.tbodyClass }, __props.tbodyTransitionProps, toHandlers(__props.tbodyTransitionHandlers, !0)), [
            renderSlot(_ctx.$slots, "top-row", { headers: headers.value }),
            displayItems.value.length ? (openBlock(!0), createElementBlock(Fragment, { key: 1 }, renderList(displayItems.value, (item, index) => renderSlot(_ctx.$slots, "row", {
              key: `item-${index}`,
              item,
              index
            }, () => [
              createElementVNode("tr", {
                class: normalizeClass([
                  typeof __props.tbodyTrClass == "function" ? __props.tbodyTrClass(item) : __props.tbodyTrClass,
                  selectedRows.value.includes(item) ? `table-${__props.selectedVariant}` : ""
                ]),
                onClick: ($event) => onRowClick(item),
                onDblclick: ($event) => _ctx.$emit("row-dblclicked", item, index, $event),
                onContextmenu: ($event) => _ctx.$emit("row-contextmenu", item, index, $event),
                onMouseenter: ($event) => _ctx.$emit("row-hovered", item, index, $event),
                onMouseleave: ($event) => _ctx.$emit("row-unhovered", item, index, $event),
                onMousedown: ($event) => _ctx.$emit("row-middle-clicked", item, index, $event)
              }, [
                (openBlock(!0), createElementBlock(Fragment, null, renderList(headers.value, (header) => renderSlot(_ctx.$slots, header.key, {
                  key: header.key,
                  field: header,
                  item,
                  index,
                  value: item[header.key]
                }, () => [
                  (openBlock(), createElementBlock("td", {
                    key: `item-${index}-${header}`
                  }, [
                    renderSlot(_ctx.$slots, `${header.key}-content`, {
                      field: header,
                      item,
                      index,
                      value: item[header.key]
                    }, () => [
                      createTextVNode(toDisplayString(item[header.key]), 1)
                    ])
                  ]))
                ])), 128))
              ], 42, _hoisted_11),
              __props.showDetails && selectedRows.value.includes(item) ? (openBlock(), createBlock(Transition, mergeProps({
                key: 0,
                ref_for: !0
              }, __props.detailsTransitionProps, toHandlers(__props.detailsTransitionHandlers)), {
                default: withCtx(() => [
                  createElementVNode("tr", null, [
                    createElementVNode("td", {
                      colspan: headers.value.length,
                      class: normalizeClass(__props.detailsTdClass)
                    }, [
                      renderSlot(_ctx.$slots, "row-details", {
                        item,
                        index
                      })
                    ], 10, _hoisted_12)
                  ])
                ]),
                _: 2
              }, 1040)) : createCommentVNode("", !0)
            ])), 128)) : renderSlot(_ctx.$slots, "no-items", {
              key: 0,
              headers: headers.value
            }, () => [
              createElementVNode("tr", null, [
                createElementVNode("td", {
                  colspan: headers.value.length,
                  class: "text-center"
                }, [
                  renderSlot(_ctx.$slots, "empty", { headers: headers.value }, () => [
                    createTextVNode(toDisplayString(__props.emptyText), 1)
                  ])
                ], 8, _hoisted_10)
              ])
            ]),
            renderSlot(_ctx.$slots, "bottom-row", { headers: headers.value })
          ], 16),
          __props.footClone ? (openBlock(), createElementBlock("tfoot", _hoisted_13, [
            createElementVNode("tr", {
              class: normalizeClass([__props.tfootTrClass, __props.footRowVariant ? `table-${__props.footRowVariant}` : ""])
            }, [
              renderSlot(_ctx.$slots, "custom-foot", {}, () => [
                (openBlock(!0), createElementBlock(Fragment, null, renderList(headers.value, (header) => (openBlock(), createElementBlock("th", {
                  key: `footer-${header.key}`,
                  class: normalizeClass({
                    "table-heading-sortable": header.sortable && !__props.noFooterSorting,
                    "table-heading-sortable-asc": sorted.value[header.key] === "asc",
                    "table-heading-sortable-desc": sorted.value[header.key] === "desc"
                  }),
                  onClick: ($event) => !__props.noFooterSorting && sort(header)
                }, [
                  renderSlot(_ctx.$slots, `footer-${header.key}`, { header }, () => [
                    createTextVNode(toDisplayString(header.label || header.key), 1)
                  ])
                ], 10, _hoisted_14))), 128))
              ])
            ], 2)
          ])) : createCommentVNode("", !0)
        ], 10, _hoisted_4)
      ], 6),
      __props.perPage > 0 ? (openBlock(), createElementBlock("div", _hoisted_15, [
        createElementVNode("div", _hoisted_16, [
          _cache[3] || (_cache[3] = createElementVNode("span", { class: "me-2" }, "Show", -1)),
          createElementVNode("select", {
            value: __props.perPage,
            class: "form-select form-select-sm",
            style: { width: "auto" },
            onChange: onPerPageChange
          }, [
            (openBlock(!0), createElementBlock(Fragment, null, renderList(__props.perPageOptions, (option) => (openBlock(), createElementBlock("option", {
              key: option,
              value: option
            }, toDisplayString(option), 9, _hoisted_18))), 128))
          ], 40, _hoisted_17),
          _cache[4] || (_cache[4] = createElementVNode("span", { class: "ms-2" }, "entries", -1))
        ]),
        createElementVNode("div", _hoisted_19, [
          _cache[5] || (_cache[5] = createElementVNode("span", { class: "me-2" }, "Page", -1)),
          createElementVNode("select", {
            value: __props.currentPage,
            class: "form-select form-select-sm",
            style: { width: "auto" },
            onChange: onPageChange
          }, [
            (openBlock(!0), createElementBlock(Fragment, null, renderList(totalPages.value, (page) => (openBlock(), createElementBlock("option", {
              key: page,
              value: page
            }, toDisplayString(page), 9, _hoisted_21))), 128))
          ], 40, _hoisted_20),
          createElementVNode("span", _hoisted_22, "of " + toDisplayString(totalPages.value), 1)
        ])
      ])) : createCommentVNode("", !0)
    ]));
  }
});
export {
  _sfc_main as default
};
//# sourceMappingURL=Table.vue.js.map
